import time


class Condition:
    class _ConditionNode:
        def reset_condition(self):
            for cond in (self.cond1, self.cond2):
                if hasattr(cond, "reset_condition"):
                    cond.reset_condition()

    class _And(_ConditionNode):
        def __init__(self, cond1, cond2):
            self.cond1 = cond1
            self.cond2 = cond2

        def __call__(self):
            return self.cond1() and self.cond2()

    class _Or(_ConditionNode):
        def __init__(self, cond1, cond2):
            self.cond1 = cond1
            self.cond2 = cond2

        def __call__(self):
            return self.cond1() or self.cond2()

    def __init__(self, cond):
        self.condition = Condition._And(cond, Condition.ALWAYS)

    def AND(self, cond):
        self.condition = Condition._And(self.condition, cond)

        return self

    def OR(self, cond):
        self.condition = Condition._Or(self.condition, cond)

        return self

    @classmethod
    def ALWAYS(cls):
        return Condition(lambda: True)

    @classmethod
    def NEVER(cls):
        return Condition(lambda: False)

    def reset_condition(self):
        self.condition.reset_condition()

    def __call__(self):
        return self.condition()

    def Timer(amt):
        return _Timer(amt)


class _Timer(Condition):
    def __init__(self, amt):
        super().__init__(self)

        self.amt = amt
        self.reset_condition()

    def reset_condition(self):
        self.end_time = time.time() + self.amt

    def __call__(self):
        return time.time() >= self.end_time



import Bullpen

import time


class Manager:
    def __init__(self, config):
        self.config = Manager._convert_transitions_to_list(config)
        self.current_action = list(self.config.keys())[0]
        self.current_transitions = self.config[self.current_action]
        self._next_action = None

    def perform(self):
        self.reset_transition_conditions()

        while not self.ready_to_rotate():
            self.current_action.on_perform()

            time.sleep(0.05)

        return self.rotate()

    def rotate(self):
        self.current_action = self._next_action
        self.current_transitions = self.config[self.current_action]
        self._next_action = None

        return self.perform()

    def ready_to_rotate(self):
        for transition in self.current_transitions:
            if transition.can_transition():
                self._next_action = transition.to

                return True

        return False

    def reset_transition_conditions(self):
        for transition in self.current_transitions:
            transition.on.reset_condition()

    @staticmethod
    def _convert_transitions_to_list(config):
        converted = {}

        for key, value in config.items():
            if isinstance(value, Bullpen.Transition):
                converted[key] = [value]
            else:
                converted[key] = value

        return converted



class Transition:
    def __init__(self, **options):
        self.to = options["to"]
        self.on = options["on"]

    def can_transition(self):
        return self.on()

    def transition(self):
        if self.can_transition():
            return self.to



#!/usr/bin/env python

from setuptools import find_packages, setup
import codecs
import os.path as path

cwd = path.dirname(__file__)

with open("description.md", "r") as fh:
    long_description = fh.read()

version = "0.0.0"
with codecs.open(path.join(cwd, "Bullpen/version.py"), "r", "ascii") as f:
    exec(f.read())
    version = __version__
assert version != "0.0.0"

setup(
    name="Bullpen",
    author="Tyler Porter",
    author_email="tyler.b.porter@gmail.com",
    version=version,
    license="MIT",
    description="A workflow management suite for arbitrary actions",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/ty-porter/Bullpen",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Other Audience",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Natural Language :: English",
        "Topic :: Other/Nonlisted Topic",
    ],
    keywords=["LED matrix", "matrix", "raspberry pi", "raspberry", "RPI", "LED"],
    platforms="ANY",
    packages=find_packages(),
    data_files=[
        ("docs", ["README.md", "LICENSE", "description.md"]),
        (
            "RGBMatrixEmulator",
            [
                "RGBMatrixEmulator/icon.ico",
                "RGBMatrixEmulator/icon.png",
                "RGBMatrixEmulator/adapters/browser_adapter/static/index.html",
                "RGBMatrixEmulator/adapters/browser_adapter/static/assets/client.js",
                "RGBMatrixEmulator/adapters/browser_adapter/static/assets/icon.ico",
                "RGBMatrixEmulator/adapters/browser_adapter/static/assets/styles.css",
            ],
        ),
    ],
    include_package_data=True,
)
